# -*- coding: utf-8 -*-
import urllib2
from bs4 import BeautifulSoup
from static import *
import traceback
import re


def _make_beautifulsoup(url):
    log("MAKE SOUP: " + url)
    req = urllib2.Request(url)
    req.add_header('User-Agent', user_agent)
    response = urllib2.urlopen(req, timeout=15)
    data = response.read()
    soup = BeautifulSoup(data.decode('utf-8', 'ignore'), "html.parser")
    response.close()
    return soup


def _fix_url(link):
    if root_url_sort not in link:
        link = root_url + link
    if link[:2] == "//":
        link = "http:" + link
    if 'thumbs/' in link:
        link = link.replace('thumbs/', '')
    return link


def _process_titles(title, otitle=None):
    year = re.search(r"(\(((\d{4})|(\d{4})-\d{4})\))", title)
    if year:
        title = title.replace(year.group(1), '')
        year = year.group(3) or year.group(4)
    season = None
    sre = re.compile(r"(\(((\d+) Sezonas|Season (\d+))\))", re.I)
    for match in sre.finditer(title):
        title = title.replace(match.group(1), '')
        season = int(match.group(3) or match.group(4))

    if otitle:
        for match in sre.finditer(otitle):
            otitle = otitle.replace(match.group(1), '')
            season = int(match.group(3) or match.group(4))
        otitle = otitle.strip().encode("utf-8")

    title = title.strip().encode("utf-8")
    if title.count('/') == 1:
        title = title[:title.index('/')].strip()

    if season and year:
        title = "{} (sezonas {}, {})".format(title, season, year)
    elif season:
        title = "{} ({} sezonas)".format(title, season)
    elif year:
        title = "{} ({})".format(title, year)
    return title, otitle, year, season


def _process_f_table(table, data, info):

    for li in table.find_all('li'):
        ftt = li.find('span', class_='f-table-title')
        if not ftt:
            continue
        ftt = ftt.getText()
        if ftt == 'Metai:' and not info.year:
            val = li.find('span', class_='f-table-text')
            info.year = None if not val else int(val.getText().replace(' ', '')[:4])

        elif ftt == u'Šalis:' and not info.studio:
            val = li.find('span', class_='f-table-text')
            info.studio = None if not val else val.getText()

        elif ftt == u'Žanras:' and not info.genre:
            val = li.find('span', class_='f-table-text')
            info.genre = None if not val else val.getText()

        elif ftt == u'Režisierius:' and not info.director:
            val = li.find('span', class_='f-table-text')
            info.director = None if not val else val.getText()

        elif ftt == 'Aktoriai:' and not info.cast:
            info.cast = []
            for a in li.find_all('a'):
                if a is not None:
                    info.cast.append(a.getText())

        elif ftt == 'Kalba:':
            val = li.find('span', class_='f-table-text')
            if val:
                if val.getText().lower() == u"anglų + lt titrai":
                    data.lang = "en"
                    data.subtitle = "lt"
                elif "Lietuv" in val.getText():
                    data.lang = "lt"
                elif "Angl" in val.getText():
                    data.lang = "en"
                elif "Rus" in val.getText():
                    data.lang = "ru"
    return data, info


def process_video_page(url):
    """
    Responsible for getting video pages in list/grid pages
    :type url: str
    :rtype: list of VideoMenu
    """
    try:
        soup = _make_beautifulsoup(_fix_url(url))
        array = []
        vid_content = soup.find(id='dle-content')
        if not vid_content:
            log("vid_content is None")
            return [array]

        # Getting videos
        for listing in vid_content.find_all('div', class_='short'):
            obj = listing.find_all('a')[0]

            # Formatting title
            titles = listing.find('h3', class_='short-title')
            title, otitle, year, season = _process_titles(titles.find('span').getText(), titles.find('small').getText())

            em = obj.find('em')
            tagline = None if not em else em.getText()

            quality = listing.find('span', class_='short-quality')
            codec = None if not quality else quality.getText()

            data = VideoMenu(
                title,
                img_url=_fix_url(obj.find('img')['src']),
                page_url=_fix_url(obj['href']),
                codec=codec,
                originaltitle=otitle,
                tagline=tagline,
                season=season,
                year=year,
            )

            s2t = listing.find('div', class_='short2-text')
            data, info = _process_f_table(s2t, data, data.info)
            data.info = info
            p = s2t.find_all('p')[-1]
            data.info.plotoutline = None if not p else p.getText()

            array.append(data)

        # Getting pages
        try:
            navi = vid_content.find('div', class_='pages')
            pages = navi.find('div', class_='pages-numbers')
            page_cur = None
            for s in pages.find_all('span'):
                if s.getText() == '...':
                    continue
                page_cur = int(s.getText())
            if not page_cur:
                raise Exception
            a = pages.find_all('a')
            if len(a) > 1:
                page_max = int(a[-1].getText())
            elif len(a) == 1:
                page_max = 2
            else:
                raise Exception
            if page_cur >= page_max:
                raise Exception
            page_nxt = navi.find('span', class_='pages-next').find('a')['href']
            log("PAGES: C:{}, M:{}, N:{}".format(page_cur, page_max, page_nxt))
            return [array, page_cur, page_max, page_nxt]
        except Exception:
            traceback.print_exc()
            return [array]
    except urllib2.HTTPError:
        pass


def progress_search(phrase, page):
    ret = process_video_page(_fix_url(search_url.format(phrase, page)))
    if len(ret) == 4:
        return [ret[0], ret[2]]
    else:
        return ret


def progress_video(url):
    """
    :type url: str
    :rtype: Video
    """
    soup = _make_beautifulsoup(_fix_url(url))
    fstory = soup.find(id='fstory')
    data = Video()
    info = VideoInfo.load_current()

    try:
        info.plot = fstory.find('div', class_='fstory-story margin-b25').getText()
    except AttributeError:
        log("Failed get plot")

    try:
        spans = fstory.find('div', class_='fstory-rating').find_all('span')
        info.votes = "Balsai: {}, reitingas: {}".format(spans[-1].getText(), spans[-2].getText())
    except AttributeError:
        log("Failed get votes")

    try:
        date = fstory.find('div', class_='bar-newsinfo').find('small').getText()
        if date.count('-') == 2:
            info.dateadded = "{2}-{1}-{0} 00:00:00".format(*date.replace(' ', '').split('-'))
    except AttributeError:
        log("Failed get dateadded")

    try:
        data.img_url = fstory.find('div', class_='fullstory-poster').find('img')['src']
    except AttributeError:
        log("Failed get thumbnail")

    if not info.title:
        title, _, year, season = _process_titles(fstory.find('h1', class_='f-padding').find('strong').getText())
        info.title = title
        info.year = year
        info.season = season

    data, info = _process_f_table(fstory.find('ul', class_="f-table"), data, info)

    for iframe in fstory.find_all('iframe'):
        try:
            # For each source
            src = iframe['src']
            try:
                if iframe['title'] == "YouTube video player":
                    data.trailer_url = src
                    continue
            except KeyError:
                pass
            is_series, source = _grab_source(src, info)
            if is_series:
                data.is_series = is_series
                data.sources = source
                break
            else:
                data.sources.append(source)

        except NotImplementedError:
            continue

    return data


def gen_random():
    soup = _make_beautifulsoup(root_url)

    list = []
    for m in soup.find_all('div', class_='short-anywhere'):
        a = m.find('a')
        data = Video()
        data.img_url = m.find('img')['src'].replace('thumbs/', '')
        data.page_url = a['href']
        data.title = a['title']
        list.append(data)
    return list


def _grab_source(url, info):
    if root_url_sort not in url:
        raise NotImplementedError

    log("Grabbing source form %s" % url)
    import json

    is_series = False
    new_soup = _make_beautifulsoup(url)
    script = new_soup.find_all('script')[-1].getText()
    titles = new_soup.find_all('title')

    if len(titles) > 0 and titles[0].getText()[-4:] == ".txt":
        # If series, get all series sources
        is_series = True
        source = []
        # Not the best approach!
        uppl = script.index("{'playlist'")
        json_data = script[uppl:script.index("}]},", uppl) + 3].replace("'", '"')
        for sscr in json.loads(json_data)['playlist']:
            linfo = VideoInfo(**info.compose())
            comment = sscr['comment']
            linfo.episode = int(re.findall('\d+', comment)[0])
            msg = "{} {}" if not isinstance(linfo.title, unicode) else u"{} {}"  # unicode nightmare
            linfo.title = msg.format(linfo.title, sscr['comment'].strip())
            source.append(Source(sscr['file'], linfo))
    else:
        # Must be a movie

        # Not the best approach either
        uppl = script.index("file:'") + 6
        source = Source(script[uppl:script.index("',", uppl)], info)
    return is_series, source
