# -*- coding: utf-8 -*-
"""
This file contains static variables
for menu mapping and urllib2.
"""
import sys
import xbmc
import xbmcaddon

debug = xbmcaddon.Addon().getSetting('debug')

__all__ = ['log', 'VideoInfo', 'Video', 'VideoMenu', 'Menu', 'Source', 'user_agent', 'root_url', 'root_url_sort',
           'search_url']
user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.0.3) Gecko/2008092417 Firefox/3.0.3'

root_url = "http://filmux.org"
root_url_sort = "filmux.org"
search_url = "/index.php?do=search&subaction=search&search_start={1}&full_search=0&result_from=13&story={0}"


def log(s):
    if debug:
        if isinstance(s, unicode):
            s = s.encode('ascii', 'replace').decode("'utf-8")
        else:
            s = s.decode('ascii', 'replace')
        print(u"FILMUX.ORG: %s" % s)


class Menu:
    Link, Menu, Random, Search, Top = range(5)

    def __init__(self, option, label):
        """
        :param option: Action once menu is selected
        :param label: Label of menu
        """
        self.option = option
        self.label = label

        if isinstance(option, str):
            self.type = Menu.Link
        elif isinstance(option, list):
            self.type = Menu.Menu
        else:
            self.type = option


class VideoInfo:
    def __init__(self, title=None, year=None, genre=None, director=None, cast=None, episode=None,
                 season=None, plot=None, originaltitle=None, votes=None, trailer=None, dateadded=None, studio=None,
                 tagline=None, plotoutline=None):
        # Header
        self.title = title  # Actual title
        self.originaltitle = originaltitle  # Title in english
        self.dateadded = dateadded
        self.year = year

        # Within content
        self.genre = genre
        self.studio = studio  # Country
        self.director = director
        self.cast = cast
        self.plot = plot
        self.plotoutline = plotoutline  # Shorter description provided in videos menu page.
        self.votes = votes

        # Additional
        self.trailer = trailer

        # Series only
        self.episode = episode
        self.season = season
        self.tagline = tagline  # Info, usually last update

    def compose(self):
        data = {}
        for key, val in self.__dict__.items():
            if val is not None:
                data[key] = val
        log("Video info: %s" % data)
        return data

    @staticmethod
    def load_current():
        info = VideoInfo()
        for key in info.__dict__.keys():
            val = unicode(xbmc.getInfoLabel("ListItem.{}".format(key)), 'utf-8')
            if val == '':
                continue
            setattr(info, key, val)
        if info.season:
            info.season = int(info.season)
        if info.episode:
            info.episode = int(info.episode)
        if info.cast:
            info.cast = info.cast.split('\n')
        return info


class VideoMenu:
    def __init__(self, title, img_url=None, codec=None, page_url=None, **info):
        self.page_url = page_url
        self.img_url = img_url
        self.codec = codec
        self.lang = None
        self.subtitle = None
        self.info = VideoInfo(title, **info)


class Video:
    def __init__(self):
        self.sources = []
        self.trailer_url = None  # raw youtube link
        self.img_url = None
        self.codec = None  # Rather just quality than codec
        self.lang = None
        self.subtitle = None
        self.is_series = False

    def add_source(self, sources):
        """
        :type sources: Source
        """
        self.sources.append(sources)

    def get_sources(self):
        """
        :rtype: list of Source
        """
        return self.sources


class Source:
    def __init__(self, url, info):
        """
        :param url: Url link to source steam
        :type info: VideoInfo
        """
        self.url = url
        self.info = info


genres_films = [
    Menu("/filmai/dramos/", "Dramos"),
    Menu("/filmai/siaubo/", "Siaubo"),
    Menu("/filmai/trileriai/", "Trileriai"),
    Menu("/filmai/mistiniai/", "Mistiniai"),
    Menu("/filmai/veiksmo/", "Veiksmo"),
    Menu("/filmai/kariniai/", "Kariniai"),
    Menu("/filmai/vesternai/", "Vesternai"),
    Menu("/filmai/komedijos/", "Komedijos"),
    Menu("/filmai/romantiniai/", "Romantiniai"),
    Menu("/filmai/istoriniai/", "Istoriniai"),
    Menu("/filmai/muzikiniai/", "Muzikiniai"),
    Menu("/filmai/nuotykiu/", "Nuotykių"),
    Menu("/filmai/seimai/", "Šeimai"),
    Menu("/filmai/fantastiniai/", "Fantastiniai"),
    Menu("/filmai/kriminaliniai/", "Kriminaliniai"),
    Menu("/filmai/dokumentiniai/", "Dokumentiniai"),
    Menu("/filmai/biografinis/", "Biografiniai"),
    Menu("/filmai/detektyviniai/", "Detektyviniai"),
    Menu("/filmai/sportas/", "Sportas"),
]

genres_series = [
    Menu("/serialai/", "Visi Serialai"),
    Menu("/serialai/drama/", "Dramos"),
    Menu("/serialai/siaubas/", "Siaubo"),
    Menu("/serialai/trileris/", "Trileriai"),
    Menu("/serialai/mistika/", "Mistiniai"),
    Menu("/serialai/veiksmas/", "Veiksmo"),
    Menu("/serialai/karinis/", "Kariniai"),
    Menu("/serialai/vesternas/", "Vesternai"),
    Menu("/serialai/komedija/", "Komedijos"),
    Menu("/serialai/romantika/", "Romantiniai"),
    Menu("/serialai/istorinis/", "Istoriniai"),
    Menu("/serialai/muzikinis/", "Muzikiniai"),
    Menu("/serialai/nuotykiai/", "Nuotykių"),
    Menu("/serialai/seimos/", "Šeimai"),
    Menu("/serialai/fantastika/", "Fantastiniai"),
    Menu("/serialai/dokumentika/", "Dokumentiniai"),
    Menu("/serialai/kriminalai/", "Kriminaliniai"),
    Menu("/serialai/detektyvai/", "Detektyviniai"),
    Menu("/serialai/sport/", "Sportas"),
    Menu("/serialai/biografiniai/", "Biografiniai"),
]

lang_movies = [
    Menu("/filmai/filmai-lt/", "Filmai Lietuviški"),
    Menu("/filmai/filmai-en/", "Filmai Angliški"),
    Menu("/filmai/filmai-ru/", "Filmai Rusiški"),
    Menu("/filmai/filmai-lt-sub/", "Filmai su subtitrais")
]
lang_series = [
    Menu("/serialai/serialai-lt/", "Serialai Lietuviškai"),
    Menu("/serialai/serialai-en/", "Serialai Angliškai"),
    Menu("/serialai/serialai-ru/", "Serilai Rusiškai"),
    Menu("/serialai/serialai-lt-sub/", "Serialai su subtitrais")
]
menu_films = [
    Menu("/filmai/", "Visi filmai"),
    Menu(genres_films, "Žandrai"),
    Menu(lang_movies, "Vertimas")
]
menu_series = [
    Menu("/serialai/", "Visi serialai"),
    Menu(genres_series, "Žandrai"),
    Menu(lang_series, "Vertimas")
]

main_menu = [
    Menu("/index.php?do=lastnews", "Naujienos"),
    Menu(menu_films, "Filmai"),
    Menu(menu_series, "Serialai"),
    Menu("/animacija/", "Animacija"),
    Menu("/tv-laidos", "TV Laidos"),
    Menu(Menu.Search, "Paieška..."),
    Menu(Menu.Random, "Atsitiktinai filmai"),
    # TODO: Add more menus
    # Menu("/top/", "TOP 250"),
    # Menu(None, "Pagal Metus"),
]
